@@ -1,5 +1,5 @@
 /* tc-s390.h -- Header file for tc-s390.c.
-   Copyright 2000, 2001 Free Software Foundation, Inc.
+   Copyright 2000, 2001, 2002 Free Software Foundation, Inc.
    Written by Martin Schwidefsky (schwidefsky@de.ibm.com).
 
    This file is part of GAS, the GNU Assembler.
@@ -17,7 +17,7 @@
    You should have received a copy of the GNU General Public License
    along with GAS; see the file COPYING.  If not, write to the Free
    Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA. */
+   02111-1307, USA.  */
 
 #define TC_S390
 
@@ -34,7 +34,7 @@ struct fix;
    are willing to perform this relocation while building the .o file.
    This is only used for pcrel relocations, so GOTOFF does not need to be
    checked here.  I am not sure if some of the others are ever used with
-   pcrel, but it is easier to be safe than sorry. */
+   pcrel, but it is easier to be safe than sorry.  */
 
 #define TC_RELOC_RTSYM_LOC_FIXUP(FIX)        \
   ((FIX)->fx_r_type != BFD_RELOC_390_GOTENT  \
@@ -44,22 +44,24 @@ struct fix;
            && S_IS_DEFINED ((FIX)->fx_addsy)      \
            && ! S_IS_COMMON ((FIX)->fx_addsy))))
 
-#define TC_FORCE_RELOCATION(FIXP)       \
-  ((FIXP)->fx_r_type == BFD_RELOC_VTABLE_INHERIT	\
-   || (FIXP)->fx_r_type == BFD_RELOC_VTABLE_ENTRY)
+#define TC_FORCE_RELOCATION(FIXP) tc_s390_force_relocation(FIXP)
+extern int tc_s390_force_relocation PARAMS ((struct fix *));
 
 #define tc_fix_adjustable(X)  tc_s390_fix_adjustable(X)
 extern int tc_s390_fix_adjustable PARAMS ((struct fix *));
 
+#define TC_FIX_ADJUSTABLE(fixP) \
+  (! symbol_used_in_reloc_p ((fixP)->fx_addsy) && tc_fix_adjustable (fixP))
+
 /* The target BFD architecture.  */
 #define TARGET_ARCH bfd_arch_s390
 extern enum bfd_architecture s390_arch PARAMS ((void));
 
 /* The target BFD format.  */
 #define TARGET_FORMAT s390_target_format()
-extern const char * s390_target_format PARAMS ((void));
+extern const char *s390_target_format PARAMS ((void));
 
-/* Set the endianness we are using. */
+/* Set the endianness we are using.  */
 #define TARGET_BYTES_BIG_ENDIAN 1
 
 /* Whether or not the target is big endian */
@@ -83,9 +85,6 @@ extern int target_big_endian;
 /* We don't need to handle .word strangely.  */
 #define WORKING_DOT_WORD
 
-/* We set the fx_done field appropriately in md_apply_fix.  */
-#define TC_HANDLES_FX_DONE
-
 #define md_number_to_chars           number_to_chars_bigendian
 
 #define md_do_align(n, fill, len, max, around)                          \
@@ -106,9 +105,6 @@ if (fragP->fr_type == rs_align_code)					\
 			   - fragP->fr_address				\
 			   - fragP->fr_fix));
 
-/* call md_apply_fix3 with segment instead of md_apply_fix */
-#define MD_APPLY_FIX3
-
 /* call md_pcrel_from_section, not md_pcrel_from */
 #define MD_PCREL_FROM_SECTION(FIXP, SEC) md_pcrel_from_section(FIXP, SEC)
 extern long md_pcrel_from_section PARAMS ((struct fix *, segT));
